# coding: utf-8

"""
    Production API

    API exposing endpoints for managing well  and daily production.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class NetworkWellConstraintInput(BaseModel):
    """
    NetworkWellConstraintInput
    """ # noqa: E501
    source_network_id: StrictStr = Field(description="Source ID of the network definition", alias="sourceNetworkId")
    source_well_id: StrictStr = Field(description="Source ID of the network well definition", alias="sourceWellId")
    source_id: StrictStr = Field(description="Source ID for the well/date constraint. This sourceID enables upsert functionality from client source systems of record", alias="sourceId")
    var_date: datetime = Field(description="Date at which the constraint becomes applicable. Datetime offset (with timezone) in ISO 8601 format. Example: 2022-09-27T16:28:37+0000", alias="date")
    max_pressure: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum pressure constraint, psig", alias="maxPressure")
    min_pressure: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum pressure constraint, psig", alias="minPressure")
    max_oil_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum oil rate constraint, STB/d", alias="maxOilRate")
    max_gas_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum gas rate constraint, Mscf/d", alias="maxGasRate")
    max_water_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum water rate constraint, STB/d", alias="maxWaterRate")
    max_liquid_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum liquid rate constraint, STB/d", alias="maxLiquidRate")
    max_drawdown: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum drawdown constraint (Reservoir pressure - bottomhole pressure), psig", alias="maxDrawdown")
    planned_downtime: Optional[Union[Annotated[float, Field(le=100, strict=True, ge=0)], Annotated[int, Field(le=100, strict=True, ge=0)]]] = Field(default=None, description="Planned downtime in percent (100% == 24 hours)", alias="plannedDowntime")
    max_gas_lift_injection_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum gas lift injection rate, Mscf/d", alias="maxGasLiftInjectionRate")
    min_gas_lift_injection_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum gas lift injection rate, Mscf/d", alias="minGasLiftInjectionRate")
    max_gas_lift_injection_pressure: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum gas lift injection pressure, psig", alias="maxGasLiftInjectionPressure")
    min_gas_lift_injection_pressure: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum gas lift injection pressure, psig", alias="minGasLiftInjectionPressure")
    __properties: ClassVar[List[str]] = ["sourceNetworkId", "sourceWellId", "sourceId", "date", "maxPressure", "minPressure", "maxOilRate", "maxGasRate", "maxWaterRate", "maxLiquidRate", "maxDrawdown", "plannedDowntime", "maxGasLiftInjectionRate", "minGasLiftInjectionRate", "maxGasLiftInjectionPressure", "minGasLiftInjectionPressure"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of NetworkWellConstraintInput from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if max_pressure (nullable) is None
        # and model_fields_set contains the field
        if self.max_pressure is None and "max_pressure" in self.model_fields_set:
            _dict['maxPressure'] = None

        # set to None if min_pressure (nullable) is None
        # and model_fields_set contains the field
        if self.min_pressure is None and "min_pressure" in self.model_fields_set:
            _dict['minPressure'] = None

        # set to None if max_oil_rate (nullable) is None
        # and model_fields_set contains the field
        if self.max_oil_rate is None and "max_oil_rate" in self.model_fields_set:
            _dict['maxOilRate'] = None

        # set to None if max_gas_rate (nullable) is None
        # and model_fields_set contains the field
        if self.max_gas_rate is None and "max_gas_rate" in self.model_fields_set:
            _dict['maxGasRate'] = None

        # set to None if max_water_rate (nullable) is None
        # and model_fields_set contains the field
        if self.max_water_rate is None and "max_water_rate" in self.model_fields_set:
            _dict['maxWaterRate'] = None

        # set to None if max_liquid_rate (nullable) is None
        # and model_fields_set contains the field
        if self.max_liquid_rate is None and "max_liquid_rate" in self.model_fields_set:
            _dict['maxLiquidRate'] = None

        # set to None if max_drawdown (nullable) is None
        # and model_fields_set contains the field
        if self.max_drawdown is None and "max_drawdown" in self.model_fields_set:
            _dict['maxDrawdown'] = None

        # set to None if planned_downtime (nullable) is None
        # and model_fields_set contains the field
        if self.planned_downtime is None and "planned_downtime" in self.model_fields_set:
            _dict['plannedDowntime'] = None

        # set to None if max_gas_lift_injection_rate (nullable) is None
        # and model_fields_set contains the field
        if self.max_gas_lift_injection_rate is None and "max_gas_lift_injection_rate" in self.model_fields_set:
            _dict['maxGasLiftInjectionRate'] = None

        # set to None if min_gas_lift_injection_rate (nullable) is None
        # and model_fields_set contains the field
        if self.min_gas_lift_injection_rate is None and "min_gas_lift_injection_rate" in self.model_fields_set:
            _dict['minGasLiftInjectionRate'] = None

        # set to None if max_gas_lift_injection_pressure (nullable) is None
        # and model_fields_set contains the field
        if self.max_gas_lift_injection_pressure is None and "max_gas_lift_injection_pressure" in self.model_fields_set:
            _dict['maxGasLiftInjectionPressure'] = None

        # set to None if min_gas_lift_injection_pressure (nullable) is None
        # and model_fields_set contains the field
        if self.min_gas_lift_injection_pressure is None and "min_gas_lift_injection_pressure" in self.model_fields_set:
            _dict['minGasLiftInjectionPressure'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of NetworkWellConstraintInput from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "sourceNetworkId": obj.get("sourceNetworkId"),
            "sourceWellId": obj.get("sourceWellId"),
            "sourceId": obj.get("sourceId"),
            "date": obj.get("date"),
            "maxPressure": obj.get("maxPressure"),
            "minPressure": obj.get("minPressure"),
            "maxOilRate": obj.get("maxOilRate"),
            "maxGasRate": obj.get("maxGasRate"),
            "maxWaterRate": obj.get("maxWaterRate"),
            "maxLiquidRate": obj.get("maxLiquidRate"),
            "maxDrawdown": obj.get("maxDrawdown"),
            "plannedDowntime": obj.get("plannedDowntime"),
            "maxGasLiftInjectionRate": obj.get("maxGasLiftInjectionRate"),
            "minGasLiftInjectionRate": obj.get("minGasLiftInjectionRate"),
            "maxGasLiftInjectionPressure": obj.get("maxGasLiftInjectionPressure"),
            "minGasLiftInjectionPressure": obj.get("minGasLiftInjectionPressure")
        })
        return _obj


