# coding: utf-8

"""
    Production API

    API exposing endpoints for managing well  and daily production.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self

class PCPInput(BaseModel):
    """
    PCPInput
    """ # noqa: E501
    source_well_id: StrictStr = Field(description="SourceWellId of the well which owns the ESP", alias="sourceWellId")
    source_wellbore_id: StrictStr = Field(description="SourceWellboreId of the wellbore which owns the ESP", alias="sourceWellboreId")
    source_tubing_id: StrictStr = Field(description="SourceTubingId of the tubing string in which the ESP is installed", alias="sourceTubingId")
    source_id: StrictStr = Field(description="SourceId of the ESP to enable upsert functionality", alias="sourceId")
    name: StrictStr = Field(description="Pump name, must match a pump in the catalog")
    viscosity_correction: Optional[StrictBool] = Field(description="Whether to apply viscosity corrections in conjunction with gas separation", alias="viscosityCorrection")
    separation_efficiency: Union[StrictFloat, StrictInt] = Field(description="Default gas separation efficiency when it cannot be calculated, %", alias="separationEfficiency")
    run_date: datetime = Field(description="Run date of the ESP. Datetime offset (with timezone) in ISO 8601 format. Example: 2022-09-27T16:28:37+0000", alias="runDate")
    pull_date: Optional[datetime] = Field(default=None, description="Pull date of the ESP. Datetime offset (with timezone) in ISO 8601 format. Example: 2022-09-27T16:28:37+0000", alias="pullDate")
    slip_factor: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Slip factor override from bench test", alias="slipFactor")
    flow_factor: Optional[Union[StrictFloat, StrictInt]] = Field(default=1.0, description="Flow factor asjustment from bench test", alias="flowFactor")
    head_factor: Optional[Union[StrictFloat, StrictInt]] = Field(default=1.0, description="Head factor adjustment from bench test", alias="headFactor")
    power_factor: Optional[Union[StrictFloat, StrictInt]] = Field(default=1.0, description="Power factor adjustment from bench test", alias="powerFactor")
    __properties: ClassVar[List[str]] = ["sourceWellId", "sourceWellboreId", "sourceTubingId", "sourceId", "name", "viscosityCorrection", "separationEfficiency", "runDate", "pullDate", "slipFactor", "flowFactor", "headFactor", "powerFactor"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PCPInput from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if viscosity_correction (nullable) is None
        # and model_fields_set contains the field
        if self.viscosity_correction is None and "viscosity_correction" in self.model_fields_set:
            _dict['viscosityCorrection'] = None

        # set to None if pull_date (nullable) is None
        # and model_fields_set contains the field
        if self.pull_date is None and "pull_date" in self.model_fields_set:
            _dict['pullDate'] = None

        # set to None if slip_factor (nullable) is None
        # and model_fields_set contains the field
        if self.slip_factor is None and "slip_factor" in self.model_fields_set:
            _dict['slipFactor'] = None

        # set to None if flow_factor (nullable) is None
        # and model_fields_set contains the field
        if self.flow_factor is None and "flow_factor" in self.model_fields_set:
            _dict['flowFactor'] = None

        # set to None if head_factor (nullable) is None
        # and model_fields_set contains the field
        if self.head_factor is None and "head_factor" in self.model_fields_set:
            _dict['headFactor'] = None

        # set to None if power_factor (nullable) is None
        # and model_fields_set contains the field
        if self.power_factor is None and "power_factor" in self.model_fields_set:
            _dict['powerFactor'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PCPInput from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "sourceWellId": obj.get("sourceWellId"),
            "sourceWellboreId": obj.get("sourceWellboreId"),
            "sourceTubingId": obj.get("sourceTubingId"),
            "sourceId": obj.get("sourceId"),
            "name": obj.get("name"),
            "viscosityCorrection": obj.get("viscosityCorrection"),
            "separationEfficiency": obj.get("separationEfficiency"),
            "runDate": obj.get("runDate"),
            "pullDate": obj.get("pullDate"),
            "slipFactor": obj.get("slipFactor"),
            "flowFactor": obj.get("flowFactor") if obj.get("flowFactor") is not None else 1.0,
            "headFactor": obj.get("headFactor") if obj.get("headFactor") is not None else 1.0,
            "powerFactor": obj.get("powerFactor") if obj.get("powerFactor") is not None else 1.0
        })
        return _obj


